#!/usr/bin/python
# -*- coding: utf-8 -*-

# moveItemToPoint 1.0b2 
# 6/28/2008 tatsuya6502
# 6/29/2008 tatsuya6502

import lx

#
# 指定したタイプのレイヤーのリストを返します
#
# @params type   main | forground | background
# @returns   レイヤーのリスト。該当無しの場合は None
#
def getLayers(type) : 
	numLayers = lx.eval("query layerservice layer.n ? %s" % type)
	
	if (numLayers == 0) : 
		return None

	# TODO: use lx.evalN
	layers = lx.eval("query layerservice layers ? %s" % type)

	if (numLayers == 1) :
		return [layers]
	else:
		return layers


#
# レイヤーIDで指定したアイテムの名前を返します
#
# @params layerID   レイヤーID
# @return レイヤー名
#
def getItemName(layerID) : 
	return lx.eval("query layerservice layer.name ? %s" % layerID)


#
# レイヤーIDで指定したアイテムを選択します
# 
# @params layerID   レイヤーID
# 
def selectItem(layerID) : 
	lx.eval("select.item \"%s\"" % getItemName(layerID))


#
# レイヤーIDで指定したアイテムのペアレントのレイヤーIDを返します
# 
# @param   layerID  レイヤーID
# @returns ペアレントのレイヤーID
#
def getParentItem(layerID) :
	parentID = lx.eval("query layerservice layer.parent ? \"%s\"" % getItemName(layerID)) 

	if (parentID >= 0) : 
		# modo 302では、layer.parentのインデックスは 0から始まるようだが、
		# layer.indexは 1から始まるため +1 する
		parentID += 1
		
		#lx.out(getItemName(parentID))
		
	return parentID	

# 
# 選択された全ての頂点を返します
#
# @returns   頂点のリスト
#
def getSelectedVertexes() :
	numVerts = lx.eval("query layerservice vert.n ? selected")

	if (numVerts == 0) : 
		return None

	# TODO: use lx.evalN
	verts = lx.eval("query layerservice verts ? selected")
	
	if (numVerts == 1) :
		return [verts]
	else: 
		return verts
	
	
#
# 指定された複数の頂点を選択します
#
# @param vertexList
#
def selectVertexes(vertexList) : 
	for vertexID in vertexList : 
		layerID = lx.eval("query layerservice vert.layer ? %s" % vertexID)

		#lx.eval("select.vertex action:add value:\"%s:%s\"" % (layerID, vertexID))
		lx.eval("select.vertex 0 0 0 \"%s:%s\"" % (layerID, vertexID))
	

#
# 指定された頂点のポジションを返します
#
# @param vertexID    頂点のID
# @returns   x, y z座標のリスト。頂点が選択されていない時は None
#
def getVertexPosition(vertexID) :
	return lx.eval("query layerservice vert.pos ? %s" % vertexID)


#
# レイヤーアイテムのポジションをセットします
#
# @params position3d   x, y z座標のリスト
#
def setItemPosition(position3d) :
	lx.eval("item.channel pos.X %s" % position3d[0])
	lx.eval("item.channel pos.Y %s" % position3d[1])
	lx.eval("item.channel pos.Z %s" % position3d[2])
	
	

# #########################################################

POINT3D_NAME = "modolab.moveItemToPoint.point3d"

def isPositionValueDefined() : 
	return lx.eval("query scriptsysservice userValue.isDefined ? %s" % POINT3D_NAME)


def savePosition(position3d) : 
	if (not isPositionValueDefined()) :
		lx.eval("user.defNew %s string temporary" % POINT3D_NAME)

	lx.eval("user.value %s \"%s %s %s\"" 
			% (POINT3D_NAME, position3d[0], position3d[1], position3d[2]))


def loadPosition() : 
	userValue = lx.eval("user.value %s ?" % POINT3D_NAME)
	
	if (userValue != None) :
		position3d = userValue.split(" ")

	return position3d

# #########################################################

#
# layerIDで指定したアイテムのペアレントがitemListにないことを
# 確認することで、アイテムのポジションを変えるべきか判断します
#
# @param layerID  
# @param itemList
# @return True:  アイテムがitemListに含まれていない
#         Flase: 含まれている
#
def shouldMoveItem(layerID, itemList) : 
	parentID = getParentItem(layerID)

	while (parentID > 0) : 
		if (parentID in itemList) : 
			return False
			
		parentID = getParentItem(parentID)

	return True


#
# position3dで指定した位置に全てのバックグラウンドアイテムを移動します
#
# @params position3d   移譲先の x, y, z座標のリスト
#
def moveBackgroundItemsToPosition(position3d) :
	items = getLayers("background")
	
	if (items == None) : 
		return
	
	for item in items :
		if (shouldMoveItem(item, items)) : 	
			selectItem(item)
			setItemPosition(position3d)


def markPosition() : 
	getLayers("forground")
	selectedVertexes = getSelectedVertexes()

	if (selectedVertexes != None) :
		position3d = getVertexPosition(selectedVertexes[0])
		savePosition(position3d)


def moveItem() : 
	position3d = loadPosition()

	if (position3d != None) : 
		setItemPosition(position3d)


#
# main
# 
args = lx.args()

if (len(args) > 0) :
	command = args[0]
	
if (command.lower() == "mark") : 
	markPosition()

elif (command.lower() == "move") :
	moveItem()
